"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MachineImage = void 0;
const utility_1 = require("@jfrog-pipelines/utility");
const errors_1 = require("@jfrog-pipelines/errors");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const BuildPlaneImageDao_1 = require("../../driven/db/BuildPlaneImageDao");
const MachineImageService_1 = require("./MachineImageService");
const MachineImageValidator_1 = require("./MachineImageValidator");
class MachineImage {
    constructor() {
        this.machineImageDao = new BuildPlaneImageDao_1.BuildPlaneImageDao();
        this.machineImageService = new MachineImageService_1.MachineImageService();
    }
    async create(machineImage) {
        MachineImageValidator_1.MachineImageValidator.validateCreate(machineImage);
        const machineImageAttribute = {
            name: machineImage.name,
            masterIntegrationId: machineImage.masterIntegrationId,
            imageId: machineImage.imageId,
            region: machineImage.region,
            isAvailable: machineImage.isAvailable,
            architecture: machineImage.architecture,
            operatingSystem: machineImage.operatingSystem,
            metadataPropertyBag: machineImage.metadataPropertyBag ?? {}
        };
        try {
            return await this.machineImageService.create(machineImageAttribute);
        }
        catch (error) {
            const databaseError = error;
            throw new errors_1.ConflictError(`failed creating records due to ${databaseError.name} ${databaseError.message}`);
        }
    }
    async update(id, machineImage) {
        const existingImage = await this.machineImageDao.findById(id);
        if (!existingImage) {
            throw new errors_1.NotFoundError(`machineImage not found for id: ${id}`);
        }
        if (existingImage.isInternal) {
            throw new errors_1.BadRequestError(`machineImage: ${existingImage.name} cannot be updated`);
        }
        MachineImageValidator_1.MachineImageValidator.validateMasterIntegrationForUpdate(existingImage.masterIntegrationId, existingImage.metadataPropertyBag);
        return this.getUpdatedModel(machineImage, existingImage);
    }
    async get(query) {
        const machineImages = await this.machineImageService.getAllBuildPlaneImages(query);
        return machineImages;
    }
    async getById(id, query) {
        const machineImages = await this.machineImageService.getBuildPlaneImageById(id, query);
        if ((0, utility_1.isEmpty)(machineImages)) {
            throw new errors_1.NotFoundError(`No buildPlane image found for id: ${id}`);
        }
        return machineImages;
    }
    async deleteById(id) {
        const image = await this.machineImageDao.findById(id);
        if (!image) {
            throw new errors_1.NotFoundError(`machineImage not found for id: ${id}`);
        }
        if (image.isInternal) {
            throw new errors_1.ForbiddenError(`machineImage: ${image.name} cannot be deleted`);
        }
        const deleteQuery = new data_access_layer_1.SequelizeQueryBuilder({}).where('id', id).build();
        await this.machineImageDao.delete(deleteQuery);
    }
    async getUpdatedModel(machineImage, existingImage) {
        existingImage.name = machineImage.name ?? existingImage.name;
        existingImage.imageId = machineImage.imageId ?? existingImage.imageId;
        existingImage.region = machineImage.region ?? existingImage.region;
        existingImage.isAvailable = machineImage.isAvailable ?? existingImage.isAvailable;
        if (machineImage.metadataPropertyBag) {
            const metadataPropertyBag = (0, utility_1.extend)(machineImage.metadataPropertyBag, existingImage.metadataPropertyBag);
            existingImage.metadataPropertyBag = metadataPropertyBag;
        }
        try {
            return await existingImage.save();
        }
        catch (error) {
            const databaseError = error;
            throw new errors_1.ConflictError(`failed updating records due to ${databaseError.name} ${databaseError.message}`);
        }
    }
}
exports.MachineImage = MachineImage;
//# sourceMappingURL=MachineImage.js.map