"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeProvisioner = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const uuid_1 = require("uuid");
const configuration_1 = require("@jfrog-pipelines/configuration");
const utility_1 = require("@jfrog-pipelines/utility");
const MachineImageService_1 = require("../machine-image/MachineImageService");
const JobDomainService_1 = require("../job/JobDomainService");
const NodeDomainService_1 = require("./NodeDomainService");
const ZERO = 0;
class NodeProvisioner {
    constructor(certificateInitializer) {
        this.logger = logger_1.LoggerFactory.getLogger('Provisioner');
        this.nodeService = new NodeDomainService_1.NodeDomainService(certificateInitializer);
        this.buildPlaneService = new JobDomainService_1.JobDomainService();
        this.properties = configuration_1.ConfigFactory.config;
        this.buildPlaneImageService = new MachineImageService_1.MachineImageService();
    }
    async provision(nodePool, requiredNodes) {
        this.logger = logger_1.LoggerFactory.getLogger(`NodeProvisioner:${nodePool.id}:${nodePool.name}`);
        try {
            const startNodeProvisionTime = Date.now();
            const nodes = await this.getRunningNodes(nodePool.id);
            this.logger.trace(`Successfully fetched ${nodes.length} node/s`);
            const availableNodes = this.filterAvailableNodes(nodes);
            this.logger.trace(`Current availableNodes count is ${availableNodes.length}`);
            const requiredNodeCount = this.calculateRequiredNodes(availableNodes, requiredNodes);
            this.logger.debug(`Required nodes count is ${requiredNodeCount}`);
            if (requiredNodeCount <= ZERO) {
                this.logger.debug(`No new nodes are required to be provisioned`);
                return;
            }
            const cachedNodes = this.filterCachedNodes(nodes);
            this.logger.trace(`Cached nodes count is ${cachedNodes.length}`);
            const cacheNodeData = {
                domain: 'nodepool',
                eventType: 'onNodeCached',
                count: cachedNodes.length,
                nodePoolName: nodePool.name
            };
            logger_1.LoggerFactory.logEvent(cacheNodeData);
            const activeNodes = this.filterActiveNodes(nodes);
            this.logger.trace(`Active nodes count is ${activeNodes.length}`);
            const requiredNewNodes = this.getRequiredNewNodesCount(nodePool, activeNodes, cachedNodes, requiredNodeCount);
            this.logger.trace(`Required ${requiredNewNodes} more nodes to be created`);
            const startedCachedNodes = await this.useCacheNodes(requiredNewNodes, cachedNodes);
            this.logger.info(`Started ${startedCachedNodes.length} nodes`);
            let provisionedNodesCount = 0;
            if (requiredNewNodes > ZERO) {
                const provisionedNewNodes = await this.provisionActiveNodes(requiredNewNodes, nodePool);
                provisionedNodesCount = provisionedNewNodes.length;
                this.logger.info(`Created ${provisionedNewNodes.length} new nodes`);
            }
            const endOfNodeProvisionTime = Date.now();
            const nodeProvisionTimeDiff = new Date(endOfNodeProvisionTime - startNodeProvisionTime);
            const provisionedNodeData = {
                domain: 'nodepool',
                eventType: 'onNodeProvisioned',
                count: startedCachedNodes.length + provisionedNodesCount,
                durationMillis: nodeProvisionTimeDiff.valueOf(),
                nodePoolName: nodePool.name
            };
            logger_1.LoggerFactory.logEvent(provisionedNodeData);
        }
        catch (error) {
            const message = `Failed to provision nodes for node pool id ${nodePool.id} due to ${(0, node_util_1.inspect)(error)}`;
            this.logger.warn(message);
            throw new Error(message);
        }
        finally {
            this.logger.trace(`Node provisioning completed.`);
        }
    }
    async provisionBeta(nodePool) {
        this.logger = logger_1.LoggerFactory.getLogger(`NodeProvisioner:${nodePool.id}:${nodePool.name}`);
        try {
            this.logger.trace('Fetching buildplane messages');
            const buildPlaneMessages = await this.buildPlaneService.getReadyMessages(nodePool.id);
            this.logger.trace(`Succesfully fetched ${buildPlaneMessages.length} buildplane messages`);
            const waitingNodes = (await this.nodeService.getWaitingNodes(nodePool.id)) ?? [];
            const buildPlaneMessagesByRuntime = (0, utility_1.groupBy)(buildPlaneMessages, 'runtime');
            const waitingNodeByRuntime = (0, utility_1.groupBy)(waitingNodes, 'runtime');
            const runtimes = Object.keys(buildPlaneMessagesByRuntime);
            for (const runtime of runtimes) {
                const buildPlaneMessages = buildPlaneMessagesByRuntime[runtime] ?? [];
                const waitingNodes = waitingNodeByRuntime[runtime] ?? [];
                this.logger.trace(`Processing ${buildPlaneMessages.length} buildplane messages for runtime: ${runtime}`);
                this.logger.trace(`${waitingNodes.length} waiting nodes exist for runtime: ${runtime}`);
                const requiredNewNodes = buildPlaneMessages.length - waitingNodes.length;
                this.logger.trace(`Required ${requiredNewNodes} more nodes to be created for runtime: ${runtime}`);
                if (requiredNewNodes > ZERO) {
                    const provisionedNewNodes = await this.provisionBetaNodes(nodePool, runtime, requiredNewNodes);
                    this.logger.trace(`Created ${provisionedNewNodes.length} new nodes for runtime: ${runtime}`);
                }
            }
        }
        catch (error) {
            const message = `Failed to provision nodes for node pool id ${nodePool.id} due to ${(0, node_util_1.inspect)(error)}`;
            this.logger.warn(message);
            throw new Error(message);
        }
        finally {
            this.logger.trace(`Node provisioning completed.`);
        }
    }
    filterCachedNodes(nodes) {
        const cachedNodes = nodes.filter((node) => {
            return node.statusCode === NodeDomainService_1.NodeStatus.cached;
        });
        return cachedNodes;
    }
    filterActiveNodes(nodes) {
        const activeNodeStatuses = new Set([
            NodeDomainService_1.NodeStatus.waiting,
            NodeDomainService_1.NodeStatus.success,
            NodeDomainService_1.NodeStatus.processing,
            NodeDomainService_1.NodeStatus.cached
        ]);
        const activeNodes = nodes.filter((node) => {
            return activeNodeStatuses.has(node.statusCode);
        });
        return activeNodes;
    }
    filterAvailableNodes(nodes) {
        const runningNodesStatuses = new Set([NodeDomainService_1.NodeStatus.waiting, NodeDomainService_1.NodeStatus.success]);
        const activeNodes = nodes.filter((node) => {
            return runningNodesStatuses.has(node.statusCode);
        });
        return activeNodes;
    }
    getRequiredNewNodesCount(nodePool, activeNodes, cachedNodes, requiredNodeCount) {
        const maxNodesToProvision = nodePool.numberOfNodes - activeNodes.length;
        const requiredNewNodes = Math.min(requiredNodeCount - cachedNodes.length, maxNodesToProvision);
        return requiredNewNodes;
    }
    async getRunningNodes(poolID) {
        return this.nodeService.getRunningNodes(poolID);
    }
    calculateRequiredNodes(availableNodes, messageCount) {
        const requiredCount = messageCount - availableNodes.length;
        return requiredCount;
    }
    async useCacheNodes(requiredCount, nodes) {
        const effectiveNodes = requiredCount < ZERO ? nodes.splice(-requiredCount) : nodes;
        this.logger.trace(`Starting ${effectiveNodes.length} cache nodes.`);
        const cachedNodePromises = effectiveNodes.map((node) => {
            return this.nodeService.applyNodeWaitingState(node);
        });
        return Promise.all(cachedNodePromises);
    }
    async provisionActiveNodes(count, nodePool) {
        const newRequiredNodes = Array.from({ length: count }).fill(null);
        const buildPlaneImage = await this.buildPlaneImageService.getBuildPlaneImageById(nodePool.buildPlaneImageId);
        const newNodePromises = newRequiredNodes.map(() => {
            const nodeRequestBody = this.getNodeCreationAttributes(nodePool, buildPlaneImage);
            return this.nodeService.createNode(nodeRequestBody);
        });
        return Promise.all(newNodePromises);
    }
    provisionBetaNodes(nodePool, runtime, count) {
        const newNodePromises = Array.from({ length: count }).map(() => {
            const nodeRequestBody = this.getNodeCreationAttributes(nodePool, {}, runtime);
            return this.nodeService.createBetaNode(nodeRequestBody, runtime);
        });
        return Promise.all(newNodePromises);
    }
    getNodeCreationAttributes(nodePool, buildPlaneImageAttributes, runtime) {
        const uid = (0, uuid_1.v4)();
        const EIGHT = 8;
        const nodePoolProviderMetadataPropertyBag = { ...nodePool.providerMetadataPropertyBag };
        const friendlyName = `node-${uid.slice(Math.max(ZERO, uid.length - EIGHT))}`;
        const sshPort = nodePoolProviderMetadataPropertyBag.sshPort ?? ZERO;
        const user = nodePoolProviderMetadataPropertyBag.sshUser ?? 'root';
        const buildPlaneVersion = this.properties.buildPlaneVersion ?? '';
        const nodeRequestBody = {
            projectId: nodePool.projectId,
            nodePoolId: nodePool.id,
            friendlyName: friendlyName,
            isOnDemand: true,
            statusCode: NodeDomainService_1.NodeStatus.waiting,
            statusLastUpdatedAt: new Date(),
            createdBy: NodeDomainService_1.nodeAccessorID,
            updatedBy: NodeDomainService_1.nodeAccessorID,
            isAutoInitialized: false,
            stopRequested: false,
            intendedBuildPlaneVersion: buildPlaneVersion,
            projectIntegrationId: nodePool.projectIntegrationId,
            sshPort: sshPort,
            sshUser: user,
            systemPropertyBag: {},
            initPropertyBag: nodePool.initPropertyBag,
            providerMetadataPropertyBag: {
                instanceSize: nodePoolProviderMetadataPropertyBag.instanceSize,
                operatingSystem: nodePool.operatingSystem,
                uniqueToken: (0, uuid_1.v4)(),
                diskSizeinGB: nodePool.diskSizeinGB,
                imageMetadata: buildPlaneImageAttributes?.metadataPropertyBag,
                imageId: buildPlaneImageAttributes?.imageId,
                region: buildPlaneImageAttributes?.region,
                zone: nodePoolProviderMetadataPropertyBag.zone,
                namespace: nodePoolProviderMetadataPropertyBag.namespace,
                storageClass: nodePoolProviderMetadataPropertyBag.storageClass,
                cpu: nodePoolProviderMetadataPropertyBag.cpu,
                memory: nodePoolProviderMetadataPropertyBag.memory,
                labels: nodePoolProviderMetadataPropertyBag.labels,
                subscriptionId: nodePoolProviderMetadataPropertyBag.subscriptionId,
                publicSshKey: nodePoolProviderMetadataPropertyBag.publicSshKey,
                subnetName: nodePoolProviderMetadataPropertyBag.subnetName,
                securityGroupName: nodePoolProviderMetadataPropertyBag.securityGroupName,
                resourceGroupName: nodePoolProviderMetadataPropertyBag.resourceGroupName,
                vnetName: nodePoolProviderMetadataPropertyBag.vnetName,
                adminUsername: nodePoolProviderMetadataPropertyBag.adminUsername,
                adminPassword: nodePoolProviderMetadataPropertyBag.adminPassword
            }
        };
        if (runtime) {
            nodeRequestBody.runtime = runtime;
        }
        return nodeRequestBody;
    }
}
exports.NodeProvisioner = NodeProvisioner;
//# sourceMappingURL=NodeProvisioner.js.map