"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const microservice_1 = require("@jfrog-pipelines/microservice");
const NodeDomainService_1 = require("../NodeDomainService");
const NodePoolService_1 = require("../../nodepool/NodePoolService");
const NodeProvisioner_1 = require("../NodeProvisioner");
const JobDomainService_1 = require("../../job/JobDomainService");
const BuildPlaneImageDao_1 = require("../../../driven/db/BuildPlaneImageDao");
const NodeProvisionerTestCases_1 = require("./NodeProvisionerTestCases");
const NUMBER_TWO = 2;
const NUMBER_THREE = 3;
const NUMBER_FOUR = 4;
const NUMBER_FIVE = 5;
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/redis', () => {
    const actualRedis = jest.requireActual('@jfrog-pipelines/redis');
    return {
        ...actualRedis,
        RedisClientFactory: {
            client: {}
        }
    };
});
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: {
            config: {
                buildPlaneVersion: 'testversion'
            }
        }
    };
});
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('../../../driven/acl/userService');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn(),
            logEvent: jest.fn(),
            getEventLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            })
        }
    };
});
jest.spyOn(internal_clients_1.AccessClientFactory, 'getClient').mockReturnValue({
    getNodeAccessToken: jest.fn().mockReturnValue('token')
});
const buildPlaneImageAttributes = {
    imageId: 'imageId',
    region: 'ap-south-1',
    metadataPropertyBag: '{}'
};
jest.spyOn(BuildPlaneImageDao_1.BuildPlaneImageDao.prototype, 'findById').mockImplementation(jest.fn().mockImplementation(() => {
    return buildPlaneImageAttributes;
}));
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce({});
describe('Test NodeProvisioner #provision', () => {
    const nodeProvisioner = new NodeProvisioner_1.NodeProvisioner(microservice_1.applicationContext);
    const nodePool = { id: 1, name: 'testpool', projectId: 1 };
    const getNodePoolMock = jest.spyOn(NodePoolService_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue(nodePool);
    afterEach(() => {
        publishMessageMock.mockClear();
        getNodePoolMock.mockClear();
    });
    it('should not provision nodes based on build plane message count', async () => {
        const nodes = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.cached },
            { friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.success }
        ];
        const getMessageCountMock = jest
            .spyOn(JobDomainService_1.JobDomainService.prototype, 'getReadyMessageCount')
            .mockResolvedValueOnce(NUMBER_TWO);
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting };
        jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeWaitingState').mockResolvedValueOnce(nodes[1]);
        const createNodeMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'createNode').mockResolvedValueOnce(node);
        const provisionResponse = await nodeProvisioner.provision(nodePool, NUMBER_FIVE);
        expect(provisionResponse).toBe(undefined);
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
        createNodeMock.mockClear();
    });
    it('should provision nodes based on build plane message count', async () => {
        const nodes = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.cached },
            { friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.success }
        ];
        const getMessageCountMock = jest
            .spyOn(JobDomainService_1.JobDomainService.prototype, 'getReadyMessageCount')
            .mockResolvedValueOnce(NUMBER_FIVE);
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting };
        const eventLoggerSpy = jest.spyOn(logger_1.LoggerFactory, 'logEvent');
        const createNodeMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'createNode').mockResolvedValueOnce(node);
        jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeWaitingState').mockResolvedValueOnce(nodes[1]);
        const provisionResponse = await nodeProvisioner.provision(nodePool, NUMBER_FIVE);
        expect(provisionResponse).toBe(undefined);
        expect(eventLoggerSpy).toHaveBeenCalledWith({
            count: 1,
            domain: 'nodepool',
            eventType: 'onNodeCached',
            nodePoolName: 'testpool'
        });
        expect(eventLoggerSpy).toHaveBeenCalledWith(expect.objectContaining({
            count: 1,
            domain: 'nodepool',
            eventType: 'onNodeProvisioned',
            nodePoolName: 'testpool'
        }));
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
        createNodeMock.mockClear();
    });
    it('should start cached nodes and create nodes based on buildPlaneMessage count', async () => {
        const nodePool = { id: 1, name: 'testpool', projectId: 1, numberOfNodes: 5 };
        const nodes = [{ friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.cached }];
        const getMessageCountMock = jest
            .spyOn(JobDomainService_1.JobDomainService.prototype, 'getReadyMessageCount')
            .mockResolvedValueOnce(NUMBER_FIVE);
        const applyNodeWaitingStateSpy = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeWaitingState')
            .mockResolvedValueOnce(nodes[0]);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting };
        const createNodeMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'createNode').mockResolvedValue(node);
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        const provisionResponse = await nodeProvisioner.provision(nodePool, NUMBER_FIVE);
        expect(provisionResponse).toBe(undefined);
        expect(applyNodeWaitingStateSpy).toHaveBeenCalledTimes(1);
        expect(createNodeMock).toHaveBeenCalledTimes(NUMBER_FOUR);
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
    });
    it.each((0, NodeProvisionerTestCases_1.getProvisionerTestCases)())('should create %s nodes and start %s nodes when given nodeLimit is %s, %s bpMsgs, %s success nodes, %s waiting nodes, %s cached, %s processing, %s failure nodes, %s stopped nodes', async (nodeCreated, nodeStarted, nodeLimit, buildPlaneMessageCount, successNodes, waitingNodes, cachedNodes, processingNodes, failureNodes, stoppedNodes) => {
        const nodePool = { id: 1, name: 'testpool', projectId: 1, numberOfNodes: nodeLimit };
        const nodes = (0, NodeProvisionerTestCases_1.getMockNodes)({
            successNodes,
            waitingNodes,
            cachedNodes,
            processingNodes,
            failureNodes,
            stoppedNodes
        });
        const getMessageCountMock = jest
            .spyOn(JobDomainService_1.JobDomainService.prototype, 'getReadyMessageCount')
            .mockResolvedValue(buildPlaneMessageCount);
        const applyNodeWaitingStateSpy = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeWaitingState')
            .mockResolvedValue(nodes[0]);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting };
        const createNodeMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'createNode').mockResolvedValue(node);
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        const provisionResponse = await nodeProvisioner.provision(nodePool, buildPlaneMessageCount);
        expect(provisionResponse).toBe(undefined);
        expect(createNodeMock).toHaveBeenCalledTimes(nodeCreated);
        expect(applyNodeWaitingStateSpy).toHaveBeenCalledTimes(nodeStarted);
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
    });
});
describe('Test NodeProvisioner #provisionBeta', () => {
    const nodeProvisioner = new NodeProvisioner_1.NodeProvisioner(microservice_1.applicationContext);
    const nodePool = { id: 1, name: 'testpool', projectId: 1 };
    it('should not provision nodes when there are no ready messages', async () => {
        const nodes = [
            { id: 1, runtime: 'runtime-2', statusCode: NodeDomainService_1.NodeStatus.waiting },
            { id: 2, runtime: 'runtime-2', statusCode: NodeDomainService_1.NodeStatus.waiting },
            { id: 2, runtime: 'runtime-3', statusCode: NodeDomainService_1.NodeStatus.waiting },
            { id: 2, runtime: 'runtime-4', statusCode: NodeDomainService_1.NodeStatus.waiting }
        ];
        const getReadyMessagesMock = jest.spyOn(JobDomainService_1.JobDomainService.prototype, 'getReadyMessages').mockResolvedValue([]);
        const getWaitingNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getWaitingNodes').mockResolvedValueOnce(nodes);
        const createBetaNodeMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'createBetaNode')
            .mockResolvedValueOnce([]);
        await nodeProvisioner.provisionBeta(nodePool);
        expect(getReadyMessagesMock).toHaveBeenCalledTimes(1);
        expect(getWaitingNodesMock).toHaveBeenCalledTimes(1);
        expect(createBetaNodeMock).toHaveBeenCalledTimes(0);
    });
    it('should provision nodes when there are ready messages', async () => {
        const nodes = [
            { id: 1, runtime: 'runtime-2', statusCode: NodeDomainService_1.NodeStatus.waiting },
            { id: 2, runtime: 'runtime-2', statusCode: NodeDomainService_1.NodeStatus.waiting },
            { id: 2, runtime: 'runtime-3', statusCode: NodeDomainService_1.NodeStatus.waiting },
            { id: 2, runtime: 'runtime-4', statusCode: NodeDomainService_1.NodeStatus.waiting }
        ];
        const buildPlaneMessages = [
            { id: 1, projectId: 1, runtime: 'runtime-1' },
            { id: 2, projectId: 1, runtime: 'runtime-1' },
            { id: 2, projectId: 1, runtime: 'runtime-2' },
            { id: 2, projectId: 1, runtime: 'runtime-5' }
        ];
        const getReadyMessagesMock = jest
            .spyOn(JobDomainService_1.JobDomainService.prototype, 'getReadyMessages')
            .mockResolvedValue(buildPlaneMessages);
        const getWaitingNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getWaitingNodes').mockResolvedValueOnce(nodes);
        const createBetaNodeMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'createBetaNode')
            .mockResolvedValue([]);
        await nodeProvisioner.provisionBeta(nodePool);
        expect(getReadyMessagesMock).toHaveBeenCalledTimes(1);
        expect(getWaitingNodesMock).toHaveBeenCalledTimes(1);
        expect(createBetaNodeMock).toHaveBeenCalledTimes(NUMBER_THREE);
    });
});
//# sourceMappingURL=NodeProvisioner.test.js.map