"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeStatsCheckHandler = void 0;
const node_util_1 = require("node:util");
const configuration_1 = require("@jfrog-pipelines/configuration");
const metadata_1 = require("@jfrog-pipelines/metadata");
const utility_1 = require("@jfrog-pipelines/utility");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const logger_1 = require("@jfrog-pipelines/logger");
const NodeDomainService_1 = require("../../domain/node/NodeDomainService");
const acl_1 = require("../../driven/acl");
const node_console_1 = require("../../domain/node-console");
const NodeDao_1 = require("../../driven/db/NodeDao");
const StatsDomainService_1 = require("../../domain/node-stats/StatsDomainService");
const REPORTED_AFTER_TIME_1_MIN_IN_MS = 60000;
class NodeStatsCheckHandler {
    constructor(command) {
        this.logger = logger_1.LoggerFactory.getLogger('NodeStatsCheckHandler');
        this.nodeStatsService = new StatsDomainService_1.StatsDomainService();
        this.nodeDao = new NodeDao_1.NodeDao();
        this.stepServiceRepository = new acl_1.StepRepository();
        this.properties = configuration_1.ConfigFactory.config;
        this.nodeLedger = new node_console_1.NodeConsoleLedgerService();
    }
    async handle() {
        const reportedAfterTime = this.getReportedAfterTime();
        try {
            const nodes = (await this.getNodesToCheck([metadata_1.WAITING, metadata_1.SUCCESS, metadata_1.PROCESSING], reportedAfterTime));
            const nodesProcessingBatch = 1;
            if (nodes.length === 0) {
                return;
            }
            await (0, utility_1.eachLimit)(nodes, nodesProcessingBatch, async (node) => {
                const isNodeActive = await this.isNodeActive(node.id, reportedAfterTime);
                if (isNodeActive) {
                    return;
                }
                await this.updateNodeStatus(node.id, NodeDomainService_1.NodeStatus.failure);
                if (!node.stepId) {
                    return;
                }
                this.logger.warn(`Updated node to failure since no node stats were reported for node id ${node.id} and nodepool id ${node.nodePoolId}`);
                const step = await this.stepServiceRepository.getStepById(node.stepId);
                const stepsQueuedToNode = await this.stepServiceRepository.getStepsQueuedToTheNode(step.affinityGroup, step.runId);
                this.updateStepLogs(stepsQueuedToNode);
                await this.stepServiceRepository.timeOutSteps(stepsQueuedToNode);
                await this.nodeLedger.push('Node Failed: The node has not reported any stats. Please reset the node to allow builds to continue on this node.', node.id);
                const stepIds = stepsQueuedToNode.map((step) => step.id);
                this.logger.warn(`timed out steps due to node failure, step id = ${stepIds.join(', ')}`);
            });
        }
        catch (error) {
            throw new Error((0, node_util_1.inspect)(error));
        }
    }
    async getNodesToCheck(statusArray, reportedAfter) {
        const queryObject = {
            limit: NodeDomainService_1.DEFAULT_FETCH_LIMIT
        };
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder(queryObject)
            .where('deletedAt', null)
            .whereWithOperator('statusCode', statusArray, data_access_layer_1.operatorsKey.$in)
            .where('stopRequested', 'false')
            .sort();
        const nodes = await this.nodeDao.findAll(queryBuilder.build());
        const result = nodes.filter((node) => new Date(node.createdAt) < new Date(reportedAfter));
        return result;
    }
    async updateNodeStatus(nodeId, status) {
        const updateData = {
            statusCode: status
        };
        const query = {
            id: nodeId
        };
        const updateQuery = {
            where: query
        };
        return this.nodeDao.update(updateData, updateQuery);
    }
    getReportedAfterTime() {
        const dateNow = new Date();
        const allowedInterval = Number(this.properties.maxNodeCheckInDelayInMinutes);
        const reportedInterval = allowedInterval * REPORTED_AFTER_TIME_1_MIN_IN_MS;
        const reportedAfter = dateNow.getTime() - reportedInterval;
        return new Date(reportedAfter).toISOString();
    }
    async isNodeActive(nodeId, reportedAfter) {
        const nodeStats = await this.nodeStatsService.getLatestNodeStatForNode(nodeId);
        if (!nodeStats || new Date(nodeStats.reportedAt) < new Date(reportedAfter)) {
            return false;
        }
        return true;
    }
    updateStepLogs(steps) {
        const logMessage = 'Step timed out due to node failure. ' +
            'This could have been caused by the node running out of memory. ' +
            'Please check the node stats and try re-triggering the step.';
        for (const step of steps) {
            void this.stepServiceRepository.postMessageToStepConsole('Error', 'Timeout', logMessage, step.id, step.pipelineId);
        }
    }
}
exports.NodeStatsCheckHandler = NodeStatsCheckHandler;
//# sourceMappingURL=NodeStatsCheckHandler.js.map